function open_mailbox($server, $userid, $password)
{
    echo "\n\n";
    echo "<H4>Opening IMAP connection to $server.</H4>";
    $handle = imap_open($server . "INBOX", $userid, $password);
    if ($handle === false)
    {
        echo "Failed to open IMAP mailbox INBOX.";
        print_error_stack();
        die();
    }
    return $handle;
}

function print_error_stack()
{
    echo "\n\n";
    echo "<H4>IMAP Error Stack</H4>";
    $errs = imap_errors();
    if ($errs)
    {
        foreach ($errs as $key=>$value)
        {
            echo "\n$key: $value";
        }
    }
}

function list_messages($handle)
{
    echo "\n\n";
    echo "<H4>Message Headers in INBOX</H4>";
    $headers = imap_headers ($handle);

    if ($headers == false)
    {
        echo "\nFailed to list messages.";
        print_error_stack();
    }
    else
    {
        foreach ($headers as $key=>$value)
        {
            echo "\n$value";
        }
    }
}

function get_message_count($handle)
{
    return imap_num_msg($handle);
}

function print_headers($handle, $num)
{
    echo "\n\n";
    echo "<H4>Message $n: Headers</H4>";
    $header_object = imap_headerinfo($handle, $num);
    $headers = get_object_vars($header_object);
    foreach ($headers as $property=>$value)
    {
        if (!is_array($value))
        {
            echo "\n$property: $value";
        }
        else
        {
            foreach ($value as $subvalue)
            {
                echo "\n$property: ";
                $subvalues = get_object_vars($subvalue);
                foreach ($subvalues as $item=>$itemvalue)
                {
                    echo "\n  $item=>$itemvalue";
                }
            }
        }
    }
}

function print_body_by_num($handle, $num)
{
    $id = get_message_id($handle, $num);
    print_body_by_id($handle, $id);
}

function print_body_by_id($handle, $id)
{
    echo "\n\n";
    $num = get_message_num($handle, $id);
    echo "<H4>Message ID $id (Number $num) Body</H4>";
    $s = imap_body($handle, $id, FT_UID);
    echo "\n$s";
}

function get_message_id($handle, $num)
{
    return imap_uid($handle, $num);
}

function get_message_num($handle, $id)
{
    return imap_msgno($handle, $id);
}

function close_mailbox($handle)
{
    echo "\n\n";
    echo "<H4>Closing IMAP connection.</H4>";
    $ok = imap_close($handle);
    if ($ok === false)
    {
        echo "\nFailed to close mailbox.";
        print_error_stack();
    }
}

?>

